class SaveSeries: 
    def __init__(self, results):
        self.data = results

    def openFile(self, baseName, name):
        name = '{}_{}_{}'.format(baseName, name, self.data["type"])
        name += ".txt" if self.data["q"] == False else '_q={}.txt'.format(self.data["q"])
        return open(name, "w+")

    def saveA(self,baseName, name="A"):
        file = self.openFile(baseName, name)
        for row in self.data["A"]:
            for result in row:
                file.write('{}, {}\n'.format(str(result[0]), str(result[1])))
        file.close()

    def saveGraphInd(self, baseName, name="graph_indicator"):
        file = self.openFile(baseName, name)
        for graphInd in self.data["graph_indicator"]:
            file.write('{}\n'.format(str(graphInd)))
        file.close()
    
    def saveEdge(self, baseName, name="edge_attributes"):
        file = self.openFile(baseName, name)
        for row in self.data["edge_attributes"]:
            for edge in row:
                file.write('{}\n'.format(str(edge)))
        file.close()
    
    def saveNode(self, baseName, name="node_labels"):
        file = self.openFile(baseName, name)
        for node in self.data["node_labels"]:
            file.write('{}\n'.format(str(node)))
        file.close()


    def saveAllTxt(
    self,
    baseName     =          "output",
    AName        =               "A", 
    graphIndName = "graph_indicator",
    edgeName     = "edge_attributes",
    nodeName     =     "node_labels"
    ):
        if AName != False:
            self.saveA(baseName, AName)
        if graphIndName != False:
            self.saveGraphInd(baseName, graphIndName)
        if edgeName != False and self.data["q"]:
            self.saveEdge(baseName, edgeName)
        if nodeName !=False:
            self.saveNode(baseName, nodeName)
        

def save(result, name):
    return SaveSeries(result).saveAllTxt(name)



class Calculate():
    # data must be in python list
    def __init__(self, data):
        self.data = data
        self.last = 0

    def hgInRow(self, rowNum):
        result = []
        row = self.data[rowNum-1]
        rowLen = len(row)
        for i in range(1, rowLen):
            alpha = 1
            # save somthing in the row!
            result.append([i+self.last, i+1+self.last])
            result.append([i+1+self.last, i+self.last])

            for j in range(2, rowLen-i+1):
                criterio = row[i+alpha-1]
                pendiente = row[i+j-1]
                if (criterio >= row[i-1]):
                    break
                elif (criterio < pendiente):
                    result.append([i+self.last, i+j+self.last])
                    result.append([i+j+self.last, i+self.last])
                    alpha = j
        self.last = max(max(result))
        return result

    def vgInRow(self, rowNum):
        result = []
        row = self.data[rowNum-1]
        rowLen = len(row)
        for i in range(1, rowLen):
            # save somthing in the row!
            result.append([i+self.last, i+1+self.last])
            result.append([i+1+self.last, i+self.last])

            criterio = row[i+1-1] - row[i-1]

            for j in range(2, rowLen-i+1):

                pendiente = (row[i+j-1] - row[i-1]) / j

                if(criterio < pendiente):
                    result.append([i+self.last, i+j+self.last])
                    result.append([i+j+self.last, i+self.last])

                    criterio = pendiente
        self.last = max(max(result))
        return result

    def qgInRow(self, rowNum, q):
        result = []
        graphInd = []
        row = self.data[rowNum-1]
        rowLen = len(row)
        Q = []
        v = [i for i in row]
        vStOrdQuantiles = []
        vQuantiles = [0 for i in range(0, rowLen)]
        MA = [[0 for i in range(0, q)] for j in range(0, q)]
        W = [[0 for i in range(0, q)] for j in range(0, q)]
        sum = 0

        for j in range(0, rowLen):
            for i in range(0, rowLen-1):
                if v[i] > v[i+1]:
                    v[i], v[i+1] = v[i+1], v[i]

        for i in range(0, q+1):
            Q += [int(((rowLen*i)/q))]

        for i in range(0, q):
            for j in range(Q[i], Q[i+1]):
                vStOrdQuantiles += [[v[j], i+1]]

        for j in range(0, rowLen):
            for i in range(0, rowLen):
                if(row[j] == vStOrdQuantiles[i][0]):
                    vQuantiles[j] = int(vStOrdQuantiles[i][1]-1)
                    vStOrdQuantiles[i][0] = 1000000000000
                    break

        for i in range(0, rowLen - 1):
            MA[vQuantiles[i]][vQuantiles[i+1]] = MA[vQuantiles[i]][vQuantiles[i+1]]+1

        for i in range(0, q):
            sum = 0
            for j in range(0, q):
                sum += MA[i][j]
            for j in range(0, q):
                W[i][j] = MA[i][j]/sum

        for i in range(0, q):
            for j in range(0, q):
                if W[i][j] != 0:
                    result.append([i+1+self.last, j+1+self.last])
                    graphInd.append(W[i][j])
        self.last = max(max(result))
        return result, graphInd
    
    def graphInd(self, q=False):
        graphIndList = []
        if q != False:
            for i in range(0, len(self.data)):
                for j in range(0, q):
                    graphIndList.append(i+1)
        else:
            for i in range(0, len(self.data)):
                for j in range(0, len(self.data[i])):
                    graphIndList.append(i+1)
        return graphIndList

    def nodeLabels(self, q=False):
        nodeLabelsList = []
        if q != False:
            for row in self.data:
                for i in range(0, q):
                    nodeLabelsList.append(i+1)
        else:
            for row in self.data:
                for i in range(0, len(row)):
                    nodeLabelsList.append(i+1)
        
        return nodeLabelsList

    def hg(self):
        result = []
        for i in range(1, len(self.data)+1):
            result.append(self.hgInRow(i))
        return {
            "A": result,
            "graph_indicator": self.graphInd(),
            "node_labels": self.nodeLabels(),
            "q": False,
            "type": "hg"
        }
    
    def vg(self):
        result = []
        for i in range(1, len(self.data)+1):
            result.append(self.vgInRow(i))
        return {
            "A": result,
            "graph_indicator": self.graphInd(),
            "node_labels": self.nodeLabels(),
            "q": False,
            "type": "vg"
        }


    def qg(self, q):
        result = []
        edgeAttr = []
        for i in range(1, len(self.data)+1):
            res = self.qgInRow(i, q)
            result.append(res[0])
            edgeAttr.append(res[1])
        return {
            "A": result,
            "graph_indicator": self.graphInd(q),
            "node_labels": self.nodeLabels(q),
            "edge_attributes": edgeAttr,
            "q": q,
            "type": "qg"
        }


def qg(data, q):
    return Calculate(data).qg(q)

def hg(data):
    return Calculate(data).hg()

def vg(data):
    return Calculate(data).vg()



from openpyxl import load_workbook, Workbook
import csv

class ReadSeries:
    def __init__(self, name="series.csv"):
        self.name = name
        if '.csv' in name:
            self.type = 'csv'
            csvFile = open(name)
            self.csv = csv.reader(csvFile)
        elif '.xls' in name:
            self.type = 'exel'
            self.sheet = load_workbook(name).active

    def cell(self, row, col):
        if self.type == 'exel':
            return self.sheet.cell(row, col).value

    def rowInArr(self, row):
        arr = []
        for i in range(1, self.len(row)+1):
            cell = self.cell(row, i)
            if cell == None:
                break
            arr += [self.cell(row, i)]
        return arr

    def dataInList(self):
        if self.type == 'csv':
            return self.csvInList()
        elif self.type == 'exel':
            return self.exelInList()

    def exelInList(self):
        data = []
        numberOfRows = self.rowsNum()
        for i in range(1, numberOfRows+1):
            data.append(self.rowInArr(i))
        return data
    
    def csvInList(self):
        data = []
        for row in self.csv:
            if row[0]:
                data.append([float(col) for col in row if col])
        return data

    def len(self, rowNum):
        length = len(self.sheet[rowNum])
        while (self.cell(rowNum, length) == None):
            length = length - 1
        return length
    
    def rowsNum(self):
        return len(tuple((self.sheet.rows)))

def importData(fileName):
    return ReadSeries(fileName).dataInList()